/*
 *  Gtk-Parted - a front-end for libparted
 *  Copyright (C) 2000, 2001  Damien Genet
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * popup menu functions
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <errno.h>

#include <parted/parted.h>
#include <gtk/gtk.h>

#include "../misc/misc.h"

#include "gped.h"
#include "fstab.h"
#include "ctreeParted.h"
#include "dialogOp.h"
#include "dialogMisc.h"
#include "popup_callbacks.h"



/*
 * CallBacks
 */



/*
 * return PedPartion attached to selected entry
 */
static PedPartition*
get_selected_part                      (GtkCTree        *ctreeParted)
{
	GtkCTreeNode	*node;
	PedPartition	*part;

	g_return_val_if_fail (ctreeParted, NULL);
	g_return_val_if_fail (GTK_IS_CTREE (ctreeParted), NULL);
	
	node = GTK_CTREE_NODE (GTK_CLIST (ctreeParted)->selection->data);
	if ( !node || !GTK_CTREE_ROW(node)->parent)
		g_assert_not_reached ();	/* this should never happen */
	
	if ( !(part = gtk_ctree_node_get_row_data (GTK_CTREE (ctreeParted), GTK_CTREE_NODE (node)) ))
		g_assert_not_reached ();	/* idem */

	return part;
}

static GtkCTreeNode*
_get_root_node                         (GtkCTreeNode    *node)
{
	GtkCTreeNode	*parent; 

	g_return_val_if_fail (node, NULL);
	
	if ( !(parent = GTK_CTREE_ROW (node)->parent) )
		return node;
	
	return _get_root_node (GTK_CTREE_NODE (parent));
}

static gpedDeviceInfo*
get_selected_devInfo                   (GtkCTree        *ctreeParted)
{
	GtkCTreeNode	*node;
	gpedDeviceInfo	*devInfo;

	g_return_val_if_fail (ctreeParted, NULL);
	g_return_val_if_fail (GTK_IS_CTREE (ctreeParted), NULL);
	
	node = GTK_CTREE_NODE (GTK_CLIST (ctreeParted)->selection->data);
	if ( !node )
		g_assert_not_reached ();	/* this should never happen */
	
	if (!GTK_CTREE_ROW (node)->parent) {	/* disk */
		if ( !(devInfo = gtk_ctree_node_get_row_data (GTK_CTREE (ctreeParted), GTK_CTREE_NODE (node)) ))
			g_assert_not_reached ();	/* idem */
	} else {				/* part */
		GtkCTreeNode	*root_node;
		
		root_node = _get_root_node (GTK_CTREE_NODE (node));

		if ( !(devInfo = gtk_ctree_node_get_row_data (GTK_CTREE (ctreeParted), GTK_CTREE_NODE (root_node)) ))
			g_assert_not_reached ();	/* idem */
	}

	return devInfo;
}

static PedDisk*
get_selected_disk                      (GtkCTree        *ctreeParted)
{
	g_return_val_if_fail (ctreeParted, NULL);
	g_return_val_if_fail (GTK_IS_CTREE (ctreeParted), NULL);

	return get_selected_devInfo (ctreeParted)->disk;
}

static gboolean
do_properties                          (gpedInfo         *info)
{
	GtkWidget	*dialog;
	PedPartition    *part;
	
	g_return_val_if_fail (info, FALSE);
	g_return_val_if_fail (info->ctreeParted, FALSE);
	
	part = get_selected_part (GTK_CTREE (info->ctreeParted));
	
	if ( !(dialog = dialogProperties_create (part, info)) )
		return FALSE;
	ui_dialog_run (GTK_DIALOG (dialog));

	return TRUE;
}

static gboolean
do_add                                 (gpedInfo         *info,
                                        PedPartitionType  type)
{
	GtkWidget	*dialog;
	PedPartition    *part;
	
	g_return_val_if_fail (info, FALSE);
	g_return_val_if_fail (info->ctreeParted, FALSE);
	
	part = get_selected_part (GTK_CTREE (info->ctreeParted));
	
	if ( !(dialog = dialogAdd_create (&part->geom, info, type)) )
		return FALSE;
	ui_dialog_run (GTK_DIALOG (dialog));
	
	
	return TRUE;
}

static gboolean
do_del                                 (gpedInfo         *info)
{
	PedPartition    *part;
	
	g_return_val_if_fail (info, FALSE);
	g_return_val_if_fail (info->fstab_info, FALSE);
	g_return_val_if_fail (info->ctreeParted, FALSE);
	
	part = get_selected_part (GTK_CTREE (info->ctreeParted));
	
	fstab_remove_entry (info->fstab_info, part);
	fstab_change_done (info);

	if ( !ped_disk_delete_partition (part->geom.disk, part) )
		return FALSE;
	
	gped_change_done (info, part->geom.disk);

	return TRUE;
}

static gboolean
do_del_all                             (gpedInfo         *info)
{
	PedDisk		*disk;
	
	g_return_val_if_fail (info, FALSE);
	g_return_val_if_fail (info->ctreeParted, FALSE);
	g_return_val_if_fail (info->fstab_info, FALSE);
	
	disk = get_selected_disk (GTK_CTREE (info->ctreeParted));
	
	fstab_remove_all_disk_entries (info->fstab_info, disk);
	fstab_change_done (info);
	
	if (!ped_disk_delete_all (disk))
		return FALSE;
	
	gped_change_done (info, disk);
	return TRUE;
}

static gboolean
do_maximise_part                       (gpedInfo         *info)
{
	PedConstraint 	*constraint;
	PedPartition    *part;
	
	g_return_val_if_fail (info, FALSE);
	g_return_val_if_fail (info->ctreeParted, FALSE);
	
	part = get_selected_part (GTK_CTREE (info->ctreeParted));
	
	constraint = ped_constraint_any (part->geom.disk);
	if (!ped_disk_maximize_partition (part->geom.disk, part, constraint)) {
		ped_constraint_destroy (constraint);
		return FALSE;
	}
	ped_constraint_destroy (constraint);
	
	gped_change_done (info, part->geom.disk);
	return TRUE;
}

static gboolean
do_minimise_ext                        (gpedInfo         *info)
{
	PedDisk		*disk;
	
	g_return_val_if_fail (info, FALSE);
	g_return_val_if_fail (info->ctreeParted, FALSE);
	
	disk = get_selected_disk (GTK_CTREE (info->ctreeParted));
	
	if (!ped_disk_minimize_extended_partition (disk))
		return FALSE;

	gped_change_done (info, disk);
	return TRUE;
}


static gboolean
do_write                               (gpedInfo         *info)
{
	g_return_val_if_fail (info, FALSE);
	
	if ( !gped_get_changes (info) )
		g_assert_not_reached ();
	
	gped_write_changes (info);
	
	return TRUE;
}

static gboolean
do_create_file_system                  (gpedInfo       *info,
                                        char           *fs_name)
{
	GtkWidget		*popup;
	PedPartition		*part;
	PedFileSystem		*fs;
	PedFileSystemType	*fs_type;
	
	g_return_val_if_fail (info, FALSE);
	g_return_val_if_fail (info->ctreeParted, FALSE);
	g_return_val_if_fail (fs_name, FALSE);
	
	part = get_selected_part (GTK_CTREE (info->ctreeParted));
	
	if ( ped_file_system_probe (&part->geom) ) {	/* already a FS */
		dialog_option	 ret;
	
		ret = ui_dialog_opt_create_and_run (_("Creating FileSystem will erase datas on partition"),
		                                    DIALOG_OK_CANCEL);
		if (ret == DIALOG_CANCEL) 
			return TRUE;
	}
	
	if ( gped_get_disk_changes (info, part->geom.disk) ) {	/* ask to write changes */
		dialog_option	 ret;
	
		ret = ui_dialog_opt_create_and_run (_("Disk changes must be written to create FileSystem !"),
		                                    DIALOG_OK_CANCEL);
		if (ret == DIALOG_CANCEL) 
			return TRUE;
		
		gped_write_disk_changes (info, part->geom.disk);
	}
	
	popup = ui_dialogPopup_create (_("Creating FileSystem,\n please wait..."));
	gtk_thread_inc_ref_count ();
	
	if ( !(fs_type = ped_file_system_type_get (fs_name)) )
		g_assert_not_reached ();
	
	
	if ( !(fs = ped_file_system_create (&part->geom, fs_type)) )
		goto error;
	
	
	if ( !ped_file_system_close (fs) )
		goto error;
	
	gtk_thread_dec_ref_count ();
	gtk_widget_destroy (GTK_WIDGET (popup));
	return TRUE;
	
error:
	gtk_thread_dec_ref_count ();
	gtk_widget_destroy (GTK_WIDGET (popup));
	return FALSE;
}

static gboolean
do_mklabel                             (gpedInfo         *info,
                                        char             *name)
{
	PedDiskType	*disk_type;
	gpedDeviceInfo	*devInfo;
	
	g_return_val_if_fail (info, FALSE);
	g_return_val_if_fail (info->ctreeParted, FALSE);
	g_return_val_if_fail (name, FALSE); 
	
	devInfo = get_selected_devInfo (GTK_CTREE (info->ctreeParted));
	
	if ( !(disk_type = ped_disk_type_get (name)) )
		return FALSE;

	if (devInfo->disk) {
		dialog_option	 ret;
	
		ret = ui_dialog_opt_create_and_run (_("Making a new Disk Label will erase the partition table"),
		                                    DIALOG_OK_CANCEL);
		if (ret == DIALOG_CANCEL) 
			return TRUE;
	
		ped_disk_close (devInfo->disk);
	}
	devInfo->disk = NULL;
	
	devInfo->disk = ped_disk_create (devInfo->dev, disk_type);
	
	gped_change_done (info, devInfo->disk);
	return TRUE;
}


static gboolean
do_CheckFS                             (gpedInfo       *info)
{
	GtkWidget	*popup;
	PedPartition	*part;
	PedFileSystem	*fs;
	
	g_return_val_if_fail (info, FALSE);
	g_return_val_if_fail (info->ctreeParted, FALSE);
	
	part = get_selected_part (GTK_CTREE (info->ctreeParted));

	if (!part->fs_type)
		g_assert_not_reached ();	/* ... */
	
	if ( gped_get_disk_changes (info, part->geom.disk) ) {	/* ask to write changes */
		dialog_option	 ret;
	
		ret = ui_dialog_opt_create_and_run (_("Disk changes must be written to create FileSystem !"),
		                                    DIALOG_OK_CANCEL);
		if (ret == DIALOG_CANCEL) 
			return TRUE;
		
		gped_write_disk_changes (info, part->geom.disk);
	}
	
	popup = ui_dialogPopup_create (_("Checking FileSystem,\n please wait..."));
	gtk_thread_inc_ref_count ();

	if ( !(fs = ped_file_system_open (&part->geom)) )
		goto error;
	
	if ( !ped_file_system_check (fs) ) {
		ped_file_system_close (fs);
		goto error;
	}
	
	if ( !ped_file_system_close (fs) )
		goto error;
	
	gtk_thread_dec_ref_count ();
	gtk_widget_destroy (GTK_WIDGET (popup));
	return TRUE;

error:
	gtk_thread_dec_ref_count ();
	gtk_widget_destroy (GTK_WIDGET (popup));
	return FALSE;
}

void
on_menu_popup_selected	               (gpedInfo         *info,
                                        guint             popup_entry)
{ 
	g_return_if_fail (info);
	g_return_if_fail (popup_entry);

	switch (popup_entry) {
		case POPUP_PROPERTIES:
			do_properties (info);
			break;
		
		case POPUP_ADD_PRIMARY:
			do_add (info, PED_PARTITION_PRIMARY);
			break;
		
		case POPUP_ADD_LOGICAL:
			do_add (info, PED_PARTITION_LOGICAL);
			break;
		
		case POPUP_ADD_EXTENDED:
			do_add (info, PED_PARTITION_EXTENDED);
			break;
		
		case POPUP_DEL:
			do_del (info);
			break;
		
		case POPUP_DEL_ALL:
			do_del_all (info);
			break;
		
		case POPUP_MAXIMISE_PART:
			do_maximise_part (info);
			break;
		
		case POPUP_MINIMISE_EXT:
			do_minimise_ext (info);
			break;
		
		case POPUP_CREATE_FS_ext2:
			do_create_file_system (info, "ext2");
			break;

		case POPUP_CREATE_FS_FAT:
			do_create_file_system (info, "FAT");
			break;

		case POPUP_CREATE_FS_linux_swap:
			do_create_file_system (info, "linux-swap");
			break;
		
		case POPUP_CHECK_FS:
			do_CheckFS (info);
			break;
		
		case POPUP_RESET_DOS:
			do_mklabel (info, "msdos");
			break;

		case POPUP_RESET_LOOP:
			do_mklabel (info, "loop");
			break;
			
		case POPUP_RESET_MAC:
			do_mklabel (info, "mac");
			break;
		
		case POPUP_RESET_PC98:
			do_mklabel (info, "pc98");
			break;
		
		case POPUP_REFRESH:
			ctreeParted_refresh (GTK_CTREE (info->ctreeParted),
			                     info);
			break;
		
		case POPUP_WRITE:
			do_write (info);
			break;
		
		case POPUP_APROPOS:
			dialogAPropos_create ();
			break;
			
		default:	/* should never arrive here */
			g_assert_not_reached ();
			break;
	}
}
